// SPDX-License-Identifier: MIT
pragma solidity ^0.8.13;

contract DutchAuction {

    struct AuctionItem {
        uint startPrice;
        uint endPrice;
        uint duration;
        uint interval;
        uint priceDecrement;
        address buyer;
    }

    uint public startTimestamp;
    AuctionItem[] public items;
    uint public currentItemIndex;
    bool public ended = false;

    address public vendor;
    bool public recovered = false;


    constructor(uint _startTimestamp, uint[] memory _startPrices, uint[] memory _endPrices, uint[] memory _durations, uint[] memory _intervals) {
        require(_startPrices.length == _endPrices.length && _endPrices.length == _durations.length && _durations.length == _intervals.length, "Invalid input lengths");

        startTimestamp = _startTimestamp;
        vendor = msg.sender;

        for (uint i = 0; i < _startPrices.length; i++) {
            AuctionItem memory newItem = AuctionItem({
                startPrice: _startPrices[i],
                endPrice: _endPrices[i],
                duration: _durations[i],
                interval: _intervals[i],
                priceDecrement: (_startPrices[i] - _endPrices[i]) / (_durations[i] / _intervals[i]),
                buyer: address(0)
            });

            items.push(newItem);
        }

        currentItemIndex = 0;
    }

    function getCurrentPrice() public returns (uint) {
        AuctionItem storage currentItem = items[currentItemIndex];

        require(currentItem.buyer == address(0), "Item already sold");
        require(block.timestamp >= startTimestamp, "Auction not started");
        while (block.timestamp >= startTimestamp + currentItem.duration) {
            if (currentItemIndex < items.length - 1) {
                currentItemIndex++;
                startTimestamp = startTimestamp + currentItem.duration;
                currentItem = items[currentItemIndex];
            } else {
                ended = true;
                break;
            }
        }
        require(!ended, "Auction ended");
        uint timePassed = block.timestamp - startTimestamp;
        uint price = currentItem.startPrice - currentItem.priceDecrement * (timePassed / currentItem.interval);

        //return (price > currentItem.endPrice) ? price : currentItem.endPrice;
        return price;
    }

    function getCurrentIndex() public returns (uint) {
        AuctionItem storage currentItem = items[currentItemIndex];

        require(currentItem.buyer == address(0), "Item already sold");
        require(block.timestamp >= startTimestamp, "Auction not started");
        while (block.timestamp >= startTimestamp + currentItem.duration) {
            if (currentItemIndex < items.length - 1) {
                currentItemIndex++;
            } else {
                ended = true;
                break;
            }
        }
        require(!ended, "Auction ended");
        return currentItemIndex;
    }

    function buy() external payable {
        require(currentItemIndex < items.length, "All items already sold");
        require(!ended, "Auction ended");
        uint price = getCurrentPrice();

        require(msg.value == price, "Invalid price");
        payable(msg.sender).transfer(msg.value);
        AuctionItem storage currentItem = items[currentItemIndex];
        currentItem.buyer = msg.sender;

        // Passer à l'article suivant s'il y en a un
        if (currentItemIndex < items.length - 1) {
            currentItemIndex++;
            startTimestamp = block.timestamp;
        } else {
            ended = true;
        }
    }

    function recoverFunds() external {
        require(msg.sender == vendor, "Only vendor can recover ether");
        require(ended, "Auction not ended");
        require(!recovered, "Ether already recovered");
        payable(msg.sender).transfer(address(this).balance);
        recovered = true;
    }

    function getAllAuctionItems() public returns (uint[] memory) {
        uint[] memory itemsPrices = new uint[](items.length);
        uint j = 0;
        uint currentIndex = getCurrentIndex();
        for (uint i = 0; i < items.length; i++) {
            if(i == currentIndex) {
                itemsPrices[j] = getCurrentPrice();
                j++;
            }
            else if (items[i].buyer == address(0)) {
                itemsPrices[j] = items[i].startPrice;
                j++;
            }
        }
        assembly {
            mstore(itemsPrices, j)
        }
        return itemsPrices;
    }

    function getStartTimestamp() public view returns (uint){
        return startTimestamp;
    }
}